//+------------------------------------------------------------------+
//|                                               RAVI FX Fisher.mq4 |
//|                         Copyright  2005, Luis Guilherme Damiani |
//|                                      http://www.damianifx.com.br |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, Luis Guilherme Damiani"
#property link      "http://www.damianifx.com.br"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 PaleVioletRed
#property indicator_color2 LimeGreen
#property indicator_color3 Silver
#property indicator_color4 Silver
//---- input parameters
extern int    MAfast           = 4;
extern int    MAslow           = 49;
extern double trigger          = 0.07;
extern bool   alertsOn         = False;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;
extern bool   Show3StateMode   = true;

//---- buffers
double RAVIfxFishBuffer[];
double LoTrigBuff[];
double HiTrigBuff[];
double LoNeutBuff[];
double HiNeutBuff[];
double trend[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,LoTrigBuff); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,HiTrigBuff); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,LoNeutBuff); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,HiNeutBuff); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,RAVIfxFishBuffer);
   SetIndexBuffer(5,trend);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
  {
   int counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for (int shift = limit; shift>=0; shift--)
   {
      double maf = iMA(NULL,0,MAfast,0,MODE_LWMA,PRICE_TYPICAL,shift);
      double mas = iMA(NULL,0,MAslow,0,MODE_LWMA,PRICE_TYPICAL,shift);
      double atf = iATR(NULL,0,MAfast,shift);
      double ats = iATR(NULL,0,MAslow,shift);
      double ma  = 0;
         if (mas!=0 && ats!=0)
	        ma = 100 * ((maf-mas)*atf)/mas/ats;
	        double IFish=(MathExp(2*ma)-1)/(MathExp(2*ma)+1);
	    
	    
         RAVIfxFishBuffer[shift] = IFish;
         LoTrigBuff[shift]       = EMPTY_VALUE;
         HiTrigBuff[shift]       = EMPTY_VALUE;
         LoNeutBuff[shift]       = EMPTY_VALUE;
         HiNeutBuff[shift]       = EMPTY_VALUE;
         trend[shift]            = trend[shift+1];
            if (RAVIfxFishBuffer[shift]> trigger)                                     trend[shift] =  1;
            if (RAVIfxFishBuffer[shift]<-trigger)                                     trend[shift] = -1;
            if (RAVIfxFishBuffer[shift]< trigger && RAVIfxFishBuffer[shift]> (-trigger)) trend[shift] =  0;
            if (Show3StateMode)
            {
               if  (trend[shift]== 1) { LoTrigBuff[shift] = Low[shift]; HiTrigBuff[shift] = High[shift]; }
               if  (trend[shift]==-1) { HiTrigBuff[shift] = Low[shift]; LoTrigBuff[shift] = High[shift]; }
               if  (trend[shift]== 0) { HiNeutBuff[shift] = Low[shift]; LoNeutBuff[shift] = High[shift]; }
            }
            else
            {
               if  (RAVIfxFishBuffer[shift]>0) { LoTrigBuff[shift] = Low[shift]; HiTrigBuff[shift] = High[shift]; }
               if  (RAVIfxFishBuffer[shift]<0) { HiTrigBuff[shift] = Low[shift]; LoTrigBuff[shift] = High[shift]; }
            }
       }
   manageAlerts();       
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1)                           doAlert(whichBar,"crossed trigger level up");
         if (trend[whichBar] == -1)                           doAlert(whichBar,"crossed trigger level down");
         if (trend[whichBar] == 0 && trend[whichBar+1] ==  1) doAlert(whichBar,"reverted from upper zone down");
         if (trend[whichBar] == 0 && trend[whichBar+1] == -1) doAlert(whichBar,"reverted from lower zone up");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," RAVI fx fisher ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," RAVI fx fisher "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

